"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.shell = exports.ShellContext = exports.spawn = exports.exec = exports.execa = void 0;
const tslib_1 = require("tslib");
const execa_1 = tslib_1.__importDefault(require("execa"));
exports.execa = execa_1.default;
const path_1 = tslib_1.__importDefault(require("path"));
const logger_1 = require("./logger");
const utils_1 = require("./utils");
const fs_1 = require("./fs");
const shellParser = require('shell-parser');
function _exec(cmd, options) {
    let [file, ...args] = shellParser(cmd);
    return execa_1.default(file, args, options);
}
function exec(commands, options) {
    if (utils_1.Is.str(commands)) {
        return _exec(commands, options);
    }
    const rets = [];
    let retsP = Promise.resolve(null);
    for (const cmd of commands) {
        retsP = retsP.then(() => {
            return _exec(cmd, options).then(r => {
                rets.push(r);
                return rets;
            });
        });
    }
    return retsP;
}
exports.exec = exec;
exports.spawn = execa_1.default;
class ShellContext {
    constructor() {
        this._cwdStack = [process.cwd()];
        this._env = {};
        this.logCommand = false;
        this.sleep = utils_1.sleep;
        this._logger = logger_1.logger;
    }
    /**
     * get current word directory
     */
    get cwd() {
        return this._cwdStack[this._cwdStack.length - 1];
    }
    /**
     * change work directory
     * @param dir
     */
    cd(dir) {
        this._cwdStack[this._cwdStack.length - 1] = path_1.default.resolve(this._cwdStack[this._cwdStack.length - 1], dir);
        return this;
    }
    /**
     * like pushd in shell
     * @param dir
     */
    pushd(dir) {
        this._cwdStack.push(path_1.default.resolve(this._cwdStack[this._cwdStack.length - 1], dir));
        return this;
    }
    /**
     * like popd in shell
     */
    popd() {
        this._cwdStack.pop();
        return this;
    }
    exec(commands, options) {
        this._logCmd(commands);
        let p = exec(commands, Object.assign({ cwd: this.cwd, env: Object.assign(Object.assign({}, process.env), this._env), stdio: 'inherit' }, options));
        // tslint:disable-next-line:no-floating-promises
        p.catch(err => {
            this._logger.error('Exec failed: ', commands);
            throw err;
        });
        return p;
    }
    /**
     * spawn file
     * @param file
     * @param args
     * @param options
     */
    spawn(file, args = [], options) {
        const command = file + ' ' + args.map(a => `"${a.replace(/"/g, '\\"')}"`).join(' ');
        this._logCmd(command);
        let p = exports.spawn(file, args, Object.assign({ cwd: this.cwd, env: Object.assign(Object.assign({}, process.env), this._env), stdio: 'inherit' }, options));
        // tslint:disable-next-line:no-floating-promises
        p.catch(err => {
            this._logger.error('Exec failed: ', command);
            throw err;
        });
        return p;
    }
    env(key, val) {
        if (arguments.length === 1) {
            return this._env[key];
        }
        if (utils_1.Is.defed(val)) {
            this._env[key] = val;
        }
        else {
            delete this._env[key];
        }
        return this;
    }
    /**
     * restart processes when file changes
     */
    monitor(dir, run, { throttle }) {
        let p = null;
        fs_1.fs.watchDir(dir, { throttle }, (event, file) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            while (p && !p.killed) {
                p && p.kill();
                yield utils_1.sleep(500);
            }
            let ret = run(this);
            if ('kill' in ret) {
                p = ret;
            }
            else {
                yield ret.then(r => {
                    p = r;
                    return;
                });
            }
        }));
    }
    /**
     * reset env to default
     */
    resetEnv() {
        this._env = {};
        return this;
    }
    _logCmd(cmd) {
        if (this.logCommand) {
            let env = Object.keys(this._env).map(k => `${k}=${this._env[k] || ''}`).join(' ');
            if (env) {
                env += ' ';
            }
            cmd = Array.isArray(cmd) ? cmd : [cmd];
            cmd.forEach(cmd => {
                this._logger.info(`$ ${env}${cmd}`);
            });
        }
    }
}
exports.ShellContext = ShellContext;
function shell(callback) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        return callback(new ShellContext());
    });
}
exports.shell = shell;
//# sourceMappingURL=exec.js.map